@@ -17,6 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -44,7 +45,7 @@
 #include "tds.h"
 #include "tdsutil.h"
 
-static char  software_version[]   = "$Id: config.c,v 1.3 2001/06/19 12:22:00 camber Exp $";
+static char  software_version[]   = "$Id: config.c,v 1.7 2001/11/25 19:11:39 brianb Exp $";
 static void *no_unused_var_warn[] = {software_version,
                                      no_unused_var_warn};
 
@@ -62,8 +63,12 @@ static void tds_config_verstr(char *tdsver, TDSCONFIGINFO *config);
 static int tds_config_boolean(char *value);
 static void lookup_host(const char *servername, const char *portname, char *ip, char *port);
 
+extern int g_append_mode;
+
 static char interf_file[MAXPATH];
 
+#define DEBUG_CONFIG 0
+
 /*
 ** tds_get_config() will fill the tds config structure based on configuration 
 ** information gathered in the following order:
@@ -84,8 +89,13 @@ TDSCONFIGINFO *config;
 	/* allocate a new structure with hard coded and build-time defaults */
 	config = tds_alloc_config();
 
+#if DEBUG_CONFIG
+	tdsdump_open("/tmp/tdsconfig.log");
+#endif
+	tdsdump_log(TDS_DBG_INFO1, "%L Attempting to read conf file\n");
 	if (! tds_read_conf_file(login->server_name, config)) {
 		/* fallback to interfaces file */
+		tdsdump_log(TDS_DBG_INFO1, "%L Failed reading conf file.  Trying interfaces\n");
 		tds_read_interfaces(login->server_name, config);
 	}
 
@@ -98,6 +108,10 @@ TDSCONFIGINFO *config;
 	/* And finally the login structure */
 	tds_config_login(config, login);
 
+#if DEBUG_CONFIG
+	tdsdump_close();
+#endif
+
 	return config;
 }
 static int tds_read_conf_file(char *server, TDSCONFIGINFO *config)
@@ -110,6 +124,7 @@ int found = 0;
 
 	in = fopen(FREETDS_SYSCONFFILE, "r");
 	if (in) {
+		tdsdump_log(TDS_DBG_INFO1, "%L Found conf file in %s reading sections\n",FREETDS_SYSCONFFILE);
 		found = tds_read_conf_sections(in, server, config);
 		fclose(in);
 	}
@@ -120,6 +135,7 @@ int found = 0;
 		sprintf(path,"%s/.freetds.conf",home);
 		in = fopen(path, "r");
 		if (in) {
+			tdsdump_log(TDS_DBG_INFO1, "%L Found conf file in %s/.freetds.conf reading sections\n",home);
 			found = tds_read_conf_sections(in, server, config);
 			fclose(in);
 		}
@@ -212,6 +228,7 @@ int found = 0;
 				s++;
 			}
 			if (!strcmp(section, &option[1])) {
+				tdsdump_log(TDS_DBG_INFO1, "%L Found matching section\n");
 				insection=1;
 				found=1;
 			} else {
@@ -250,12 +267,30 @@ int found = 0;
 				if (atoi(value)) 
 					config->connect_timeout = atoi(value);
 			} else if (!strcmp(option,TDS_STR_HOST)) {
+				tdsdump_log(TDS_DBG_INFO1, "%L Found host entry %s\n",value);
    				lookup_host(value, NULL, tmp, NULL);
 				if (config->ip_addr) free(config->ip_addr);
 				config->ip_addr = strdup(tmp);
+				tdsdump_log(TDS_DBG_INFO1, "%L IP addr is %s\n",config->ip_addr);
 			} else if (!strcmp(option,TDS_STR_PORT)) {
 				if (atoi(value)) 
 					config->port = atoi(value);
+			} else if (!strcmp(option,TDS_STR_EMUL_LE)) {
+				config->emul_little_endian = tds_config_boolean(value);
+			} else if (!strcmp(option,TDS_STR_TEXTSZ)) {
+				if (atoi(value)) 
+					config->text_size = atoi(value);
+			} else if (!strcmp(option,TDS_STR_CHARSET)) {
+				if (config->char_set) free(config->char_set);
+				config->char_set = strdup(value);
+			} else if (!strcmp(option,TDS_STR_CLCHARSET)) {
+				if (config->client_charset) free(config->client_charset);
+				config->client_charset = strdup(value);
+			} else if (!strcmp(option,TDS_STR_LANGUAGE)) {
+				if (config->language) free(config->language);
+				config->language = strdup(value);
+			} else if (!strcmp(option,TDS_STR_APPENDMODE)) {
+				g_append_mode = tds_config_boolean(value);
 			}
 		}
 
@@ -285,7 +320,7 @@ char ip_addr[255], ip_port[255], tds_ver[255];
 }
 static void tds_config_login(TDSCONFIGINFO *config, TDSLOGIN *login)
 {
-	if (strlen(login->server_name)) {
+	if (login->server_name && strlen(login->server_name)) {
 		if (config->server_name) free(config->server_name);
 		config->server_name = strdup(login->server_name);
 	}	
@@ -293,14 +328,52 @@ static void tds_config_login(TDSCONFIGINFO *config, TDSLOGIN *login)
 		config->major_version = login->major_version;
 		config->minor_version = login->minor_version;
 	}
-        if (strlen(login->language)) {
+        if (login->language && strlen(login->language)) {
 		if (config->language) free(config->language);
 		config->language = strdup(login->language);
 	}
-        if (strlen(login->char_set)) {
+        if (login->char_set && strlen(login->char_set)) {
 		if (config->char_set) free(config->char_set);
 		config->char_set = strdup(login->char_set);
 	}
+        if (login->host_name && strlen(login->host_name)) {
+		if (config->host_name) free(config->host_name);
+		config->host_name = strdup(login->host_name);
+		/* DBSETLHOST and it's equivilants are commentary fields
+		** they don't affect config->ip_addr (the server) but they show
+		** up in an sp_who as the *clients* hostname.  (bsb, 11/10) 
+		/* should work with IP (mlilback, 11/7/01) */
+		/*
+		if (config->ip_addr) free(config->ip_addr);
+		config->ip_addr = calloc(sizeof(char),18);
+		lookup_host(config->host_name, NULL, config->ip_addr, NULL);
+		*/
+	}
+        if (login->app_name && strlen(login->app_name)) {
+		if (config->app_name) free(config->app_name);
+		config->app_name = strdup(login->app_name);
+	}
+        if (login->user_name && strlen(login->user_name)) {
+		if (config->user_name) free(config->user_name);
+		config->user_name = strdup(login->user_name);
+	}
+        if (login->password && strlen(login->password)) {
+		if (config->password) free(config->password);
+		config->password = strdup(login->password);
+	}
+        if (login->library && strlen(login->library)) {
+		if (config->library) free(config->library);
+		config->library = strdup(login->library);
+	}
+        if (login->encrypted) {
+		config->encrypted = 1;
+	}
+        if (login->suppress_language) {
+		config->suppress_language = 1;
+	}
+        if (login->bulk_copy) {
+		config->bulk_copy = 1;
+	}
         if (login->block_size) {
 		config->block_size = login->block_size;
 	}
@@ -325,7 +398,7 @@ static void tds_config_env_tdsdump(TDSCONFIGINFO *config)
 {
 char *s;
 char path[255];
-pid_t pid;
+pid_t pid=0;
 
         if (s=getenv("TDSDUMP")) {
                 if (!strlen(s)) {
@@ -375,6 +448,10 @@ static void tds_config_verstr(char *tdsver, TDSCONFIGINFO *config)
 		config->major_version=7;
 		config->minor_version=0;
 		return;
+	} else if (!strcmp(tdsver,"80") || !strcmp(tdsver,"8.0")) {
+		config->major_version=8;
+		config->minor_version=0;
+		return;
 	}
 }
 int set_interfaces_file_loc(char *interf)
@@ -409,15 +486,6 @@ static void lookup_host(
    struct servent   *service = NULL;
    int               num     = 0;
 
-   if (portname) {
-   	 service = getservbyname(portname, "tcp");
-      if (service==NULL) {
-         num = atoi(portname);
-      } else {
-         num = ntohs(service->s_port);
-      }
-   }
-
 /* froy@singleentry.com 12/21/2000 */
 	if (host==NULL) {
 		char addr [4];
@@ -436,6 +504,15 @@ static void lookup_host(
       struct in_addr *ptr = (struct in_addr *) host->h_addr;
       strncpy(ip, inet_ntoa(*ptr), 17);
    }
+   if (portname) {
+   	 service = getservbyname(portname, "tcp");
+      if (service==NULL) {
+         num = atoi(portname);
+      } else {
+         num = ntohs(service->s_port);
+      }
+   }
+
    if (num==0) {
 	if (port) port[0] = '\0';
    } else {
@@ -443,8 +520,22 @@ static void lookup_host(
    }
 } /* lookup_host()  */
 
-
-
+static int hexdigit(char c)
+{
+	if (c>='a' && c<='f') {
+		return c - 'a' + 10;
+	} else if (c>='A' && c<='F') {
+		return c - 'A' + 10;
+	} else if (c>='0' && c<='9') {
+		return c - '0';
+	} else {
+		return 0; /* bad hex digit */
+	}
+}
+static int hex2num(char *hex)
+{
+	return hexdigit(hex[0])*16 + hexdigit(hex[1]);	
+}
 /* ========================= search_interface_file() =========================
  *
  * Def:  Open and read the file 'file' searching for a logical server
@@ -464,90 +555,97 @@ static void search_interface_file(
    char       *ip_port, /* (O) Port number for database server           */
    char       *tds_ver) /* (O) Protocol version to use when connecting   */
 {
-   char  *pathname;
-   char  line[255];
-   char  tmp_ip[sizeof(line)];
-   char  tmp_port[sizeof(line)];
-   char  tmp_ver[sizeof(line)];
-   FILE *in;
-   char *field;
-   int   found=0;
+char  *pathname;
+char  line[255];
+char  tmp_ip[sizeof(line)];
+char  tmp_port[sizeof(line)];
+char  tmp_ver[sizeof(line)];
+FILE *in;
+char *field;
+int   found=0;
 
-   ip_addr[0]  = '\0';
-   ip_port[0]  = '\0';
-   line[0]     = '\0';
-   tmp_ip[0]   = '\0';
-   tmp_port[0] = '\0';
-   tmp_ver[0]  = '\0';
+	ip_addr[0]  = '\0';
+	ip_port[0]  = '\0';
+	line[0]     = '\0';
+	tmp_ip[0]   = '\0';
+	tmp_port[0] = '\0';
+	tmp_ver[0]  = '\0';
 
-   pathname = (char *) malloc(strlen(dir) + strlen(file) + 10);
+	tdsdump_log(TDS_DBG_INFO1, "%L Searching interfaces file %s/%s\n",dir,file);
+	pathname = (char *) malloc(strlen(dir) + strlen(file) + 10);
    
-   /*
-    * create the full pathname to the interface file
-    */
-   if (file==NULL || file[0]=='\0')
-   {
-      pathname[0] = '\0';
-   }
-   else
-   {
-      if (dir==NULL || dir[0]=='\0')
-      {
-         pathname[0] = '\0';
-      }
-      else
-      {
-         strncpy(pathname, dir, sizeof(pathname));
-         strncat(pathname, "/", sizeof(pathname));
-      }
-      strncat(pathname, file, sizeof(pathname));
-      pathname[sizeof(pathname)-1] = '\0';
-   }
+	/*
+	* create the full pathname to the interface file
+	*/
+	if (file==NULL || file[0]=='\0') {
+		pathname[0] = '\0';
+	} else {
+		if (dir==NULL || dir[0]=='\0') {
+			pathname[0] = '\0';
+		} else {
+			strcpy(pathname, dir);
+			strcat(pathname, "/");
+		}
+		strcat(pathname, file);
+	}
 
 
-   /*
-    *  parse the interfaces file and find the server and port
-    */
-   if ((in = fopen(pathname,"r"))==NULL)
-   {
-      free(pathname);
-      return;
-   }
+	/*
+	*  parse the interfaces file and find the server and port
+	*/
+	if ((in = fopen(pathname,"r"))==NULL) {
+		free(pathname);
+		return;
+	}
+	tdsdump_log(TDS_DBG_INFO1, "%L Interfaces file opened\n");
 
-   while (fgets(line,sizeof(line)-1,in))
-   {
-      if (line[0]=='#') continue; /* comment */
+	while (fgets(line,sizeof(line)-1,in)) {
+		if (line[0]=='#') continue; /* comment */
 
-      if (!isspace(line[0]))
-      {
-         field = strtok(line,"\n\t ");
-         if (!strcmp(field,host)) found=1;
-         else found=0;
-      }
-      else if (found && isspace(line[0]))
-      {
-         field = strtok(line,"\n\t ");
-         if (field!=NULL && !strcmp(field,"query"))
-         {
-            field = strtok(NULL,"\n\t "); /* tcp */
-            field = strtok(NULL,"\n\t "); /* ether */
-	    strcpy(tmp_ver,field);
-            field = strtok(NULL,"\n\t "); /* host */
-
-            strcpy(tmp_ip,field);
-            field = strtok(NULL,"\n\t "); /* port */
-            strcpy(tmp_port,field);
-         }
-      }
-   }
-   fclose(in);
-   free(pathname);
+		if (!isspace(line[0])) {
+			field = strtok(line,"\n\t ");
+			if (!strcmp(field,host)) {
+				found=1;
+				tdsdump_log(TDS_DBG_INFO1, "%L Found matching entry.\n");
+			}
+			else found=0;
+		} else if (found && isspace(line[0])) {
+			field = strtok(line,"\n\t ");
+			if (field!=NULL && !strcmp(field,"query")) {
+				field = strtok(NULL,"\n\t "); /* tcp or tli */
+				if (!strcmp(field,"tli")) {
+					tdsdump_log(TDS_DBG_INFO1, "%L TLI service.\n");
+					field = strtok(NULL,"\n\t "); /* tcp */
+					field = strtok(NULL,"\n\t "); /* device */
+					field = strtok(NULL,"\n\t "); /* host/port */
+					if (strlen(field)>=18) {
+						sprintf(tmp_port,"%d", hex2num(&field[6])*256 + 
+							hex2num(&field[8]));
+						sprintf(tmp_ip,"%d.%d.%d.%d", hex2num(&field[10]),
+							hex2num(&field[12]), hex2num(&field[14]),
+							hex2num(&field[16]));
+					}
+				} else {
+					field = strtok(NULL,"\n\t "); /* ether */
+					strcpy(tmp_ver,field);
+					field = strtok(NULL,"\n\t "); /* host */
+					strcpy(tmp_ip,field);
+					tdsdump_log(TDS_DBG_INFO1, "%L host field %s\n",tmp_ip);
+					field = strtok(NULL,"\n\t "); /* port */
+					strcpy(tmp_port,field);
+				} /* else */
+			} /* if */
+		} /* else if */
+	} /* while */
+fclose(in);
+free(pathname);
 
 
    /*
     * Look up the host and service
     */
    lookup_host(tmp_ip, tmp_port, ip_addr, ip_port);
+   tdsdump_log(TDS_DBG_INFO1, "%L Resolved IP %s\n",ip_addr);
    strcpy(tds_ver,tmp_ver);
 } /* search_interface_file()  */
 
